__title__ = "pyonepassword"
__version__ = "3.11.0"
__summary__ = "A python API to query a 1Password account using the 'op' command-line tool"

"""
See PEP 440 for version scheme
https://www.python.org/dev/peps/pep-0440/#examples-of-compliant-version-schemes
Examples:

FINAL
0.9
0.9.1
0.9.2
...
0.9.10
0.9.11
1.0
1.0.1
1.1
2.0
2.0.1
...

PRE_RELEASES
X.Y.dev0 # development release 0
x.y.dev1 # development release 1
X.YaN    # Alpha release
X.YbN    # Beta release
X.YrcN   # Release Candidate
X.Y      # Final release
"""
